//###################################################################################
// File: dostest.c
//
// Mehrere Dateien auf einen frisch formatierten CF schreiben.
// Es werden immer nur 250kB pro Datei geschrieben.
//
// Zweck: Ermitteln der Schreibgeschwindigkeit wenn in der FAT
// nach freien Clustern gesucht werden mu.
//
//#########################################################################
// Last change: 25.09.2006
//#########################################################################
// hk@holger-klabunde.de
// http://www.holger-klabunde.de/index.html
//#########################################################################
// Compiler: AVR-GCC 3.4.5
//#########################################################################
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>

#include "mydefs.h"
#include "protos.h"
#include "../lcd.h"
#include "../dos.h"
#include "../printf.h"
#include "../serial.h"

//"volatile" fr alle Variablen die in Interrupt's gendert werden
unsigned int volatile countclock;

//###################################################################################
// timeout not used !
// interrupt every 47.4ms  @ 11.0592MHz
// interrupt every 32.77ms @ 16.0000MHz
SIGNAL(SIG_OVERFLOW1)        // signal handler for timer1 overflow interrupt
//###################################################################################
{
 countclock++;
}

//###################################################################################
unsigned char WriteTestFile(char *name)
//###################################################################################
{
 unsigned char result;
 unsigned long bytecount;

 bytecount=0;
 result=Fopen(name,F_WRITE);
 if(result==F_OK)
  {
   do
    {
     if(Fwrite("Hello World. Hello World. Hello World. Hello World.\n",52)!=52) result=F_ERROR;
     if(result!=F_ERROR) bytecount+=(unsigned long)52;

    }while(bytecount < (unsigned long)250000 && result==F_OK );

   Fclose();
  }

 return result;
}

//###################################################################################
/* Hauptprogramm */
int main(void)
//###################################################################################
{
 unsigned char loop;
 
 DDRC=0xFF;  	// set io-pins on PortC
 PORTC=0; 	//LCD-Datenbus

 DDRD=0xF2;  	// set io-pins on PortD
 PORTD=0x02; 	//LCD-Steuersignale siehe lcd.h

 DDRA=0; 	//Alles Eingnge
 PORTA=0xFF; 	//Eingnge mit Pullups setzen

 SFIOR&=!(1<<PUD); 	//Pullups Enable
// SFIOR=0; 	//Pullups Enable

 LCDInit();

#ifdef MMC_CARD_SPI //SD_CARD_SPI too !
 MMC_IO_Init();
#endif

 ser_init();

//Timer1
 TCCR1A=0; 
 TCCR1B=0x02;        //Timer1 Start Takt = F/8 => 47.4ms at 11.059MHz
 TIMSK=(1<<TOIE1);   // enable timer1 int

 sei();              //enable interrupts

 LCDCls();
 LCDWrite("FAT DOS-Write-Test2");
 LCDPos(2,1);
 LCDWrite("by Holgi");
 Delay1ms(1000);
 LCDCls();

 if(GetDriveInformation()!=F_OK) // get drive parameters
  {
   LCDCls();
   LCDWrite("Flash meldet sich nicht !");
   while(1);
  }
  
 RED_ON();
 GREEN_OFF();

//Ein paar Verzeichnisse/Unterverzeichnisse erzeugen
 Mkdir("dir1");
 Chdir("dir1");
 Mkdir("dir3"); // dir1/dir3
 Chdir("/");    //Back to the roots
 Mkdir("dir2");
 Chdir("dir2");
 Mkdir("dir4"); // dir2/dir4
 Chdir("/");    //Back to the roots

// Testroutine 2
 countclock=0;
 loop=0;
 for(;;)
  {
   LCDPos(1,1); LCDWrite("0.txt");
   if(WriteTestFile("0.txt")!=F_OK) break;
   Chdir("dir1");
   LCDPos(1,1); LCDWrite("1.txt");
   if(WriteTestFile("1.txt")!=F_OK) break;
   Chdir("dir3");
   LCDPos(1,1); LCDWrite("3.txt");
   if(WriteTestFile("3.txt")!=F_OK) break;
   Chdir("..");
   LCDPos(1,1); LCDWrite("1.txt");
   if(WriteTestFile("1.txt")!=F_OK) break;
   Chdir("..");
   LCDPos(1,1); LCDWrite("0.txt");
   if(WriteTestFile("0.txt")!=F_OK) break;
   Chdir("dir2");
   LCDPos(1,1); LCDWrite("2.txt");
   if(WriteTestFile("2.txt")!=F_OK) break;
   Chdir("dir4");
   LCDPos(1,1); LCDWrite("4.txt");
   if(WriteTestFile("4.txt")!=F_OK) break;
   Chdir("..");
   LCDPos(1,1); LCDWrite("2.txt");
   if(WriteTestFile("2.txt")!=F_OK) break;
   Chdir("..");
   LCDPos(1,1); LCDWrite("0.txt");
   if(WriteTestFile("0.txt")!=F_OK) break;

   loop++;
   LCDPos(2,1);
   ShowHex(loop);
  }
  
 GREEN_ON();
 RED_OFF();
 cli(); // Disable interrupts
 LCDPos(2,1);
 ShowHex((unsigned char)(countclock>>8));
 ShowHex((unsigned char)(countclock));
 
 for(;;) // loop forever
  {
  }
}

