//###################################################################################
// File: doswrite3.c
//
// 2MB groe Dateien auf den CF schreiben. Jeweils mit unterschiedlichen
// Puffergren. Zeit messen und anzeigen.
//
//#########################################################################
// Last change: 15.10.2006
//#########################################################################
// hk@holger-klabunde.de
// http://www.holger-klabunde.de/index.html
//#########################################################################
// Compiler: AVR-GCC 3.4.5
//#########################################################################
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>

#include "mydefs.h"
#include "protos.h"
#include "../lcd.h"
#include "../dos.h"
#include "../printf.h"
#include "../serial.h"

void StartTimer(void);
void StopTimer(void);

//"volatile" fr alle Variablen die in Interrupt's gendert werden
unsigned long volatile countclock;

//###################################################################################
SIGNAL(SIG_OVERFLOW1)        // signal handler for timer1 overflow interrupt
//###################################################################################
{
// TCNT1 = 65535 - ((F_CPU / 64) / 1000 ); //Counter Reload for 1ms
 TCNT1 = 65535 - ((F_CPU / 64) / 10 ); //Counter Reload for 100ms

 countclock++;
}

//###################################################################################
/* Hauptprogramm */
int main(void)
//###################################################################################
{
 unsigned long i;
 unsigned char result;
 unsigned char buf[]="Hello World. Hello World. Hello World. Hello World. Hello World.\n";
  
 DDRC=0xFF;  	// set io-pins on PortC
 PORTC=0; 	//LCD-Datenbus

 DDRD=0xF2;  	// set io-pins on PortD
 PORTD=0x02; 	//LCD-Steuersignale siehe lcd.h

 DDRA=0; 	//Alles Eingnge
 PORTA=0xFF; 	//Eingnge mit Pullups setzen

 SFIOR&=!(1<<PUD); 	//Pullups Enable
// SFIOR=0; 	//Pullups Enable

 LCDInit();

#ifdef MMC_CARD_SPI //SD_CARD_SPI too !
 MMC_IO_Init();
#endif

 ser_init();

 sei();              //enable interrupts

 LCDCls();
 puts("FAT DOS-Write-Test3");
 LCDPos(2,1);
 puts(" by Holgi\n");
 Delay1ms(1000);
 LCDCls();

 if(GetDriveInformation()!=F_OK) // get drive parameters
  {
   LCDCls();
   puts("Flash meldet sich nicht !");
   while(1);
  }
  
 LCDCls();

 RED_ON();
 GREEN_OFF();

 result=Fopen("01.TXT",F_WRITE);
 if(result==F_OK)
  {
   puts("01 ");
   StartTimer();
   for(i=0; i<((unsigned long)2000*1024); i++)
    {
     if(Fwrite(buf,1)!=1) result=F_ERROR;
     if(result==F_ERROR) break;
    }
   Fclose();
   StopTimer();
  }

 result=Fopen("02.TXT",F_WRITE);
 if(result==F_OK)
  {
   puts("02 ");
   StartTimer();
   for(i=0; i<((unsigned long)2000*1024); i+=2)
    {
     if(Fwrite(buf,2)!=2) result=F_ERROR;
     if(result==F_ERROR) break;
    }
   Fclose();
   StopTimer();
  }

 result=Fopen("04.TXT",F_WRITE);
 if(result==F_OK)
  {
   puts("04 ");
   StartTimer();
   for(i=0; i<((unsigned long)2000*1024); i+=4)
    {
     if(Fwrite(buf,4)!=4) result=F_ERROR;
     if(result==F_ERROR) break;
    }
   Fclose();
   StopTimer();
  }

 result=Fopen("08.TXT",F_WRITE);
 if(result==F_OK)
  {
   puts("08 ");
   StartTimer();
   for(i=0; i<((unsigned long)2000*1024); i+=8)
    {
     if(Fwrite(buf,8)!=8) result=F_ERROR;
     if(result==F_ERROR) break;
    }
   Fclose();
   StopTimer();
  }

 result=Fopen("16.TXT",F_WRITE);
 if(result==F_OK)
  {
   puts("16 ");
   StartTimer();
   for(i=0; i<((unsigned long)2000*1024); i+=16)
    {
     if(Fwrite(buf,16)!=16) result=F_ERROR;
     if(result==F_ERROR) break;
    }
   Fclose();
   StopTimer();
  }

 result=Fopen("32.TXT",F_WRITE);
 if(result==F_OK)
  {
   puts("32 ");
   StartTimer();
   for(i=0; i<((unsigned long)2000*1024); i+=32)
    {
     if(Fwrite(buf,32)!=32) result=F_ERROR;
     if(result==F_ERROR) break;
    }
   Fclose();
   StopTimer();
  }

 result=Fopen("64.TXT",F_WRITE);
 if(result==F_OK)
  {
   puts("64 ");
   StartTimer();
   for(i=0; i<((unsigned long)2000*1024); i+=64)
    {
     if(Fwrite(buf,64)!=64) result=F_ERROR;
     if(result==F_ERROR) break;
    }
   Fclose();
   StopTimer();
  }

 GREEN_ON();
 RED_OFF();


 for(;;) // loop forever
  {
  }
}

void StartTimer(void)
{
 countclock=0;
//Timer1 Configuration
// TCNT1 = 65535 - ((F_CPU / 64) / 1000 ); //Counter Reload for 1ms
 TCNT1 = 65535 - ((F_CPU / 64) / 10 ); //Counter Reload for 100ms

 TCCR1A=0; 
 TCCR1B=((1<<CS11) | (1<<CS10) ); // prescaler 1/64
 TIMSK|=(1<<TOIE1);   // enable timer1 int
}

void StopTimer(void)
{
 unsigned long tmp1,tmp2;
 float kBps;

 TIMSK &= ~(1<<TOIE1);   // disable timer1 int

/*
 LCDPos(1,1);
 printf("%lX ",FileCurrentCluster);

 LCDPos(1,10);
 printf("%lu ",FileCurrentSector);

 LCDPos(1,20);
 printf("%lu\n",FileSize);

 LCDPos(2,1); //Program running time
*/

 tmp1 = countclock / 10;
 tmp2 = countclock % 10;
 printf("%lu.%lus ",tmp1,tmp2);

// LCDPos(2,10);
 // We have to use float here to avoid rounding
 kBps = (float)FileSize / (float)countclock; //Bytes per 0.1 seconds
 kBps *= 100.0; //Bytes per 10 seconds to get xxx.x values
 kBps /= 1024.0; // kiloBytes per 10 seconds
 tmp1 = (unsigned int)(kBps / 10.0);
 tmp2 = (unsigned int)kBps % 10;
 printf("%lu.%lukB/s\n",tmp1,tmp2);
}
