//######################################################################
// File: display.c
//
//######################################################################
// Last change: 27.08.2006
//#########################################################################
// hk@holger-klabunde.de
// http://www.holger-klabunde.de/index.html
//#########################################################################
// Compiler: AVR-GCC 3.4.3
//#########################################################################
#include <avr/io.h>
#include <avr/interrupt.h>

#include "mydefs.h"
#include "../printf.h"
#include "../lcd.h"

#include "display.h"
#include "vs1001.h"

volatile unsigned char update_display; //down counter for update periods of the LCD display

//#########################################################################
void DisplayInit(void)
//#########################################################################
{
 LCDInit();
 update_display=UPDATE_DISPLAY;
}

//#########################################################################
void UpdateDisplay(void)
//#########################################################################
{
 int minutes,seconds;
 unsigned char sample_rate,mpgmode;
 unsigned int audata,hdat0,playtime;

 //Don't update the display EVERY time we come here
 if(update_display==0)
  {
   //Show playtime for this song
   playtime=VS1001_GetPlayTime();
   minutes=playtime/60;
   seconds=playtime%60;
   LCDPos(2,1);
   printf("%02d:%02d ",minutes,seconds);

   //get MPEG mode from VS1001
   hdat0=VS1001_GetHdat0();
   mpgmode=(unsigned char)((hdat0>>6) & 0x03); //read VS1001 datasheet
 
   audata=VS1001_GetAudata();
   sample_rate=(unsigned char)((audata>>9) & 0x0F); //read VS1001 datasheet
   audata &= 0x01FF;
   printf("% 3dkbps ",audata); //bitrate, read VS1001 datasheet
     
   switch(sample_rate)
    {
     case 1 : printf("44"); break;
     case 2 : printf("48"); break;
     case 3 : printf("32"); break;
     case 4 : printf("22"); break;
     case 5 : printf("24"); break;
     case 6 : printf("16"); break;
     case 7 : printf("11"); break;
     case 8 : printf("12"); break;
     case 9 : printf(" 8"); break;
     default: printf("??"); break;
    } 
 
   printf("kHz ");

   switch(mpgmode)
    {
     case  0 : printf("ST"); break; //Stereo
     case  1 : printf("JO"); break; //Joint Stereo
     case  2 : printf("DU"); break; //Dual ??
     case  3 : printf("MO"); break; //Mono
    }

    printf("\n");
    update_display=UPDATE_DISPLAY; //refresh update display counter
 
  }//if(update_display==0)
}
