//######################################################################
// File: playmp3.c
//
//######################################################################
// Last change: 01.09.2006
//#########################################################################
// hk@holger-klabunde.de
// http://www.holger-klabunde.de/index.html
//#########################################################################
// Compiler: AVR-GCC 3.4.5
//#########################################################################
#include <avr/io.h>
#include <avr/interrupt.h>

#include "mydefs.h"
#include "../dos.h"

#include "playmp3.h"
#include "buttons.h"
#include "rc5.h"
#include "vs1001.h"
#include "display.h"

//###################################################################################
// MP3 playing routine with control buttons checking and display output
int PlayMp3(char *name)
//###################################################################################
{
 unsigned long tmpsector;

#ifdef USE_FAT32
 unsigned long tmp;
#else
 unsigned int tmp;
#endif

 unsigned char *p;
 unsigned char secloop;
 unsigned int i;
 unsigned char result;

 if(FindName(name)==FULL_MATCH)
  {
   tmp=FileFirstCluster;
   result=F_OK;
  
   //#####################
   //begin of playing loop  
   //#####################

   while(tmp < endofclusterchain) // read all clusters of file
    {
     tmpsector=GetFirstSectorOfCluster(tmp);

     if(play_status==PLAYER_PLAY) //read nothing if playing is paused
      {

       // read all sectors of this cluster
       for(secloop=0; secloop<secPerCluster; secloop++)
        {
         ReadSector(tmpsector,iob);

         // read data from iob now
         for(i=0; i<BYTE_PER_SEC; i+=32)
          {
           p=&iob[i];          //pointer to io buffer

           //Fill VS1001 buffer until DREQ is set to 0 or end of file reached
           while(VS1001_BUSY);

           if(FileSize<32)
            {
//             for(j=(unsigned char)FileSize; j<32; j++) *p[j]=0;
             result=F_ERROR; //end of file reached
            } 
         
           if(FileSize>0) VS1001_WriteData32(p); //give VS1001 something to play
         
           FileSize-=32; // count bytes read

           if(result==F_ERROR) break;
          } //for(i=

         tmpsector++; // next sector of cluster

         if(result==F_ERROR) break;
        }// for(secloop=0; secloop<secPerCluster; secloop++)

      }//if(play_status==PLAYER_PLAY)  

     //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     //VS1001 buffer is full here, so we have some time to do other things
     //like testing input keys, remote control or update display informations.
     //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

     CheckRemote();
     CheckButtonsInLoop();

     UpdateDisplay();

     if(play_status==PLAYER_PLAY) //read nothing if playing is paused
      {
       tmp=GetNextClusterNumber(tmp);
      }//if(play_status==PLAYER_PLAY)  

     //read file to the end or STOP or NEXT or PREV or REPLAY
     if(result==F_ERROR) break;
     if(play_status!=PLAYER_PLAY && play_status!=PLAYER_PAUSE) break;
     
    }// while(tmp < endofclusterchain)

   //#####################
   //end of playing loop  
   //#####################

   VS1001_Reset(SOFT_RESET); //very important ! soft reset vs1001k after each track

   CheckButtonsAfterLoop();

   return 0; //no errors
  }//if(FindName(name)==FULL_MATCH)

 return 1; //error
}
